#!/usr/bin/env python
# -*- coding: utf-8 -*-
#  
#  License: MIT
#  
#  Copyright (c) 2019 Joerg Beckers

__all__ = ["commandmapping", "dataacquisition", "serviceshutdownhandling", "stoppablehttpserver"]